// Struc.h  Structures for Eulers and printing	

#include <stdio.h>
#include <fstream>
#include <iostream>
#include <iomanip>

using namespace std;

#ifndef __STRUC_H__
#define __STRUC_H__

struct euler 
{
	double alpha;		// Alpha angle
	double beta;		// Beta angle
	double gamma;		// Gamma angle
	double weight;		// Weight of list
};

void print_euler (const euler* start, const euler* stop)
{
  const euler * current = start;
  while (current != stop) {
    cout << fixed << setw(8) << setprecision(5) << current->alpha << "\t" 
	          << setw(8) << setprecision(5) << current->beta  << "\t" 
                  << setw(8) << setprecision(7) << current->weight << endl;
    ++current;     // increment pointer
  }
}

// count lines in file exclude spaces and #
int genV ( const char *filename ) 
{
  fstream fin(filename);
  string line;
  int i = 0;
  int k = 0;
  double a, b;

while(getline(fin, line))
  {
  if(line[0] == '#' | line.empty() ) continue; // skip '#' and empty lines in shape.txt file
  if(line[0] == '&' ) {k++;}; // skip '#' and empty lines in shape.txt file
  if(line[0] == '&' ) continue ; // skip '#' and empty lines in shape.txt file
  stringstream ss(line);
  if (ss >> a >> b) { }
  i++;
  }
  return i;
}

// count lines in file exclude spaces and #
int genV2 ( const char *filename ) 
{
  fstream fin(filename);
  string line;
  int i = 0;
  int k = 0;
  double a, b;

while(getline(fin, line))
  {
  if(line[0] == '#' | line.empty() ) continue; // skip '#' and empty lines in shape.txt file
  if(line[0] == '&' ) {k++;}; // skip '#' and empty lines in shape.txt file
  if(line[0] == '&' ) continue ; // skip '#' and empty lines in shape.txt file
  stringstream ss(line);
  if (ss >> a >> b) { }
  i++;
  }
  return k;
}


struct spectrum
{
	double freal;		// Real FID   
	double fimag;		// Imag FID   
};

struct params
{
	double start;		// first point
	double end;		// last point
	double lb;		// Line Broadening rads
	double swp;		// Sweep width ppm
	double dw ;		// Dwell time     
	double w0 ;		// Dwell time     
	double wrt ;		// vr * t     
	double wr  ;		// vr         
	double dip ;		// vr * t     
	double time;		// vr * t     
	double transmit ;	// Dwell time     
	double ph ;		// phase in rads
	double b1 ;		// applied b1 field
	double tf ;		// pulse length
	double om1 ;		// frequency?     
	double R1  ;		// R1 relaxation  
	double R2  ;		// R2 relaxation  

};

struct poly 
{
	cdouble a;		// Poly coeff
	cdouble b;		// Poly coeff
	cdouble c;		// Poly coeff
	cdouble d;		// Poly coeff 
};


void print_fid (const double* start, const double* stop)
{
  const double * current = start;
  while (current != stop) {
    //cout << fixed << setw(8) << setprecision(5) << (*current) << endl; 
    cout << *current << endl; 
    ++current;     // increment pointer
  }
}

void genFFT ( const params nmr_params, const spectrum * FID )
  {

  int end = nmr_params.end; 
  double FI , FR , FRM , sum , sum_real , sum_imag , phase , xaxis ;
  double du = nmr_params.swp /(end);
  double dw=1/(nmr_params.swp);

for(double k = -(end)/2; k < (end-1)/2; k++){
  FI=FR=FRM=sum=sum_real=sum_imag=0;
for(int m = 0; m < end; m++){
  phase = M_PI * k * (m+0.5) / end;
  FI = FID[m].fimag * ( sin ( phase ) ) ;
  FR = FID[m].freal * ( cos ( phase ) ) ;
  sum_imag = FI + sum_imag ;
  sum_real = FR + sum_real ;
  sum = sum_real + sum_imag ;
} 
  sum = ( sum / end ) ;
  sum_real = sum_real / end ;
  sum_imag = sum_imag / end ;
  xaxis = ( k * du + nmr_params.transmit ) ;
  
  cout << fixed << setw(10) << setprecision(5) << xaxis        
       << fixed << setw(10) << setprecision(5) << sum << endl; 

  sum = sum_real = sum_imag = phase = FR = FI = 0 ;
  }
}

void print_dip ( const params nmr_params, const spectrum * FID )
  {
  int end = nmr_params.end; 

for(int m = 0; m < end; m++){
    cout << fixed << setw(14) << setprecision(9) << FID[m].freal / FID[0].freal<< endl; 
  }
}

void print_fid_real ( const params nmr_params, const spectrum * FID )
  {
  int end = nmr_params.end; 
  double xaxis ;
  double dw=1/(nmr_params.swp);

for(int m = 0; m < end; m++){
  xaxis = m * dw ;
  cout << fixed << setw(10) << setprecision(5) << xaxis        
       << fixed << setw(14) << setprecision(9) << FID[m].freal << endl; 
  }
}

void print_fid_imag ( const params nmr_params, const spectrum * FID )
  {
  int end = nmr_params.end; 
  double xaxis ;
  double dw=1/(nmr_params.swp);

for(int m = 0; m < end; m++){
  xaxis = m * dw ;
  cout << fixed << setw(10) << setprecision(5) << xaxis        
       << fixed << setw(14) << setprecision(9) << FID[m].fimag << endl; 
  }
}

void print_cart ( double (*arr)[3][3] )
{
 for (int i = 0; i < 3 ; i++) {
 for (int j = 0; j < 3 ; j++) {
    cout << fixed << setw(8) << setprecision(1) << (*arr)[i][j] << " " ;
    }
  cout << endl;
  }
}

void print_cart_complex ( cdouble (*arr)[3][3] )
{
 for (int i = 0; i < 3 ; i++) {
 for (int j = 0; j < 3 ; j++) {
    cout << fixed << setw(8) << setprecision(1) << (*arr)[i][j] << " " ;
    }
  cout << endl;
  }
}

// count lines in file exclude spaces and #
// store nmr params b1 and ph
int genShape ( params * param, const char *filename ) 
{
  fstream fin(filename);
  string line;
  int i = 0;
  double a, b;

while(getline(fin, line))
  {
  if(line[0] == '#' | line.empty() ) continue; // skip '#' and empty lines in shape.txt file
  stringstream ss(line);
  if (ss >> a >> b) { }
  param[i].b1 = a;
  param[i].ph  = b;
//  cout << param[i].b1 << " " << param[i].ph << endl;
  i++;
  }
  return i;
}







#endif
