// Type.h		Various typedef declarations
// Kenji Sugase				10/9/2012

#ifndef __TYPES_h__
#define __TYPES_h__

#include <vector>
#include <string>
#include <complex> 

using namespace std;

typedef vector<int>			ivect;
typedef vector<vector<int> >		i2vect;
typedef vector<vector<vector<int> > >	i3vect;
typedef vector<double>			dvect;
typedef vector<vector<double> >		d2vect;
typedef vector<string>			svect;
typedef vector<vector<string> >		s2vect;
typedef vector<bool>			bvect;
typedef vector<vector<bool> >		b2vect;
typedef complex<int>			cint;
typedef complex<double>			cdouble;

#ifndef HZ2RADS                                 // Define Hz to Radians factor once so we'll
#define HZ2RADS 6.28318530717958647692 		// always have it around
#endif

// Trick to allow type promotion below
// allows multiply imaginary number i * an integer
// eg) i*5     without error.  otherwise need i*5.  to call a double
template <typename T>
struct identity_t { typedef T type; };

/// Make working with std::complex<> nubmers suck less... allow promotion.
#define COMPLEX_OPS(OP)                                                 \
  template <typename _Tp>                                               \
  std::complex<_Tp>                                                     \
  operator OP(std::complex<_Tp> lhs, const typename identity_t<_Tp>::type & rhs) \
  {                                                                     \
    return lhs OP rhs;                                                  \
  }                                                                     \
  template <typename _Tp>                                               \
  std::complex<_Tp>                                                     \
  operator OP(const typename identity_t<_Tp>::type & lhs, const std::complex<_Tp> & rhs) \
  {                                                                     \
    return lhs OP rhs;                                                  \
  }
COMPLEX_OPS(+)
COMPLEX_OPS(-)
COMPLEX_OPS(*)
COMPLEX_OPS(/)
#undef COMPLEX_OPS

#endif
