#include <stdio.h>
#include <iomanip>
#include <iostream>
#include <sstream>
#include <math.h>
#include <complex.h>
#include <vector>
#include <string.h>
#include "Type.h"
#include "Struc.h"
#include "Readvec.h"

using namespace std;

// Defines the point structure
  struct point 
    {
      double x;
      double y; 
      int k;
    };

int main (int argc, char *argv[])
{

        if (argc < 3) {
                cerr << 
		 "##############################################\n"
		 "##############################################\n\n"
                 "ERROR: Not enough input files specified.\n\n"
		 "USAGE: mkIntegral_cpp peak.list 2d.txt  \n\n"
		 "reads in file called scale.txt too      \n\n"   
		 "##############################################\n"
		 "##############################################\n"
		;
        }

////////////////////////////////////////////////////////////////////////////////////////////////

  const char* FILE_NAME = "scale.txt";                                 	// define file to data file to read-in
  std::vector< std::vector<double> > A = fileIn( FILE_NAME );          	// push data file in matrix A / see Read.h 
  //int A_row = A.size();					// number of rows in vector 
  int A_col = A[0].size() ;				// number of cols in vector

  //const char * filename2 = "peak.list"; 		// filename eg) shape.txt
  const char * filename2 = argv[1]; 			// filename eg) peak.list
  int Q = genV(filename2) ;				// get number of lines for allocation
  params peaks[Q] ;

  // compare peak.list and scale.txt elements to see if they match in number
  if ( Q > A_col ) {
    cerr << 
	"###############################################################\n"
	" not enough scaling elements in first row of \"scale.txt\" file \n"
	" to match \"peak.list\" components \n"
	"###############################################################\n"
	;
  return 0;
  }
 
  cout << endl << "Reading \"scale.txt\" file. " << endl << endl ;

////////////////////////////////////////////////////////////////////////////////////////////////
// loop over 2D.txt file 

  const char * filename = argv[2]; 		// filename eg) 2d.txt

  vector<point> points ;
  fstream fin(filename);
  string line;
  int i = 0, k = 0 ;
  double x, y, dummy ;

while(getline(fin, line))
  {
    if(line[0] == '#' | line.empty() ) continue;        // skip '#' and empty lines in shape.txt file
    if(line[0] == '&' ) ( k++ ) ;                       // skip '#' and empty lines in shape.txt file
    if(line[0] == '&' ) continue;                       // skip '#' and empty lines in shape.txt file
    stringstream ss(line);
    if (ss >> x >> y  ) { }
    points.push_back(point{ x, y, k });
  }

////////////////////////////////////////////////////////////////////////////////////////////////
// loop over peak.list file

  string line2;
  const char delim = ':';
  ifstream myfile (filename2);

  if (myfile.is_open())
  {
    while ( getline ( myfile, line2 ) )
    {
    //if(line2[0] == '#' | line.empty() ) continue;        // skip '#' and empty lines in shape.txt file
    if(line2[0] == '#' ) continue;                       // skip '#' and empty lines in shape.txt file
    if(line2[0] == line.empty() ) continue;                       // skip '#' and empty lines in shape.txt file
    std::istringstream is( line2 );
	int x ;
        is >> x ;
        string y;
        string z;
        getline (is, z, ':');
        is >> y >> z ;
        double yd = stof(y);
        double zd = stof(z);

////////////////////////////////////////////////////////////////////////////////////////////////
//     sort peaks from low to high just  in case they are put in backwards

    if(yd > zd ) { 
	peaks[x].start = zd ;
	peaks[x].end = yd   ; } // sort ppm range from low to high
    if (yd < zd ) { 
	peaks[x].start = yd ;
	peaks[x].end = zd   ; } // sort ppm range from low to high
    if (yd == zd ) { 
	peaks[x].start = yd ;
	peaks[x].end = zd   ; } // sort ppm range from low to high

	//cout << x << " " << peaks[x].start << " " << peaks[x].end << endl ;
    }
    myfile.close();
  }

  else cout << "peak.list file not found" << endl;

////////////////////////////////////////////////////////////////////////////////////////////////
// loop over shape field / phase

  int I = points.size();
  int M = k;
  int K = k;
  double S = I/M ;
  double add[Q][K] = {} ;

  for(int q = 0; q < Q; q++){	// peaks
  for (int m = 0; m < k ; m++) {
  for (double i = m*S  ; i < (m+1)*S   ; i++) {
	if ( points[i].x < peaks[q].end && points[i].x > peaks[q].start ) {
	     add[q][m] = add[q][m] + points[i].y ;
	} 
      }
    }
  }

////////////////////////////////////////////////////////////////////////////////////////////////

  ofstream file4;
  file4.open("dat.txt");
  double scale_conc = A[1][1] / A[1][0] ;    				// conc/intensity

  for(int k = 0; k < K; k++){	// spectrum number
	file4 << fixed << setw(5) << setprecision(1) << k * A[2][0] << " "  ;
  for(int q = 0; q < Q; q++){	// peaks
	file4 << fixed << setw(10) << setprecision(9)  ;
	file4 <<  scale_conc * add[q][k] / A[0][q] << " " ;
    }
	file4 << endl;
  }
  file4.close();

  cout << "Saving \"dat.txt\" file used in kinetics simulations " << endl << endl ;

//  print_dat (A) ;		// print out "scale.txt" file contents to screen

////////////////////////////////////////////////////////////////////////////////////////////////

    return 0;
  }




