// Simulating:  
// A -> B 
//
#include <boost/array.hpp>
#include <boost/numeric/odeint.hpp>
#include <vector>
#include <iomanip>
#include <fstream>
#include <vector>
#include <cmath>
#include <iostream>
#include <sstream>
#include "Read.h"
#include <dlib/optimization.h>

using namespace std;
using namespace boost::numeric::odeint;

typedef boost::array< double , 2 > state_type;		// number of states/peaks + 1 for time column (start from 0) 

  ofstream file3("sim.txt");                            // output file "sim.txt" for simulated fits

// passing params k1,k2 here and then calling odeint 
struct ode4
{
    double k1  	;

    ode4( double m_k1 = 1.0  )
    : k1( m_k1 )  { }

	void operator()( const state_type &x , state_type &dxdt , double t ) 
	{
        // here are the diffeqs from the rxn mechanism:
          
    	dxdt[0] = -k1 * x[0] ;	
    	dxdt[1] =  k1 * x[0] ;	

	}
} ;

//print statement to file "sim.txt"
void write_fit( const state_type &x , const double t )
{
  
  //double a = t * 10 ;
  //double b = floor(t * 10) ;		// floor statement only to print 0.1 units of time increments
  double a = t * 1 ;
  double b = floor(t * 1) ;		// floor statement only to print 1.0 units of time increments
  if (abs(b-a) < 0.001 ){          	// statement to correct for rounding errors b4 proceeding 
  file3 << fixed << setw(8) << setprecision(6) <<  t << ' ' << x[0]  
						     << ' ' << x[1]
					             << endl;
  }
}

int main(int argc, char **argv)
{
  double t_dw = 0.01 		;					// time step used in runge-kutta stepping

  const char* FILE_NAMEB = "guess.txt";                                 // define file to data file to read-in
  std::vector< std::vector<double> > k = fileIn( FILE_NAMEB );          // push data file in matrix A / see Read.h 
  cout << endl << " Reading \"guess.txt\" file for: " << endl ;
  cout << " initial rate constants " << endl ;
  cout << " initial states " << endl ;
  cout << " begin_time end_time " << endl << endl ;
  print_dat (k) ;							// print out guess.txt contents

  state_type x = { k[1][0] , k[1][1] }	; 	// initial conditions/state (peak heights at time = 0 )

  typedef runge_kutta4< state_type > stepper_type;
  integrate_const( stepper_type() , ode4( k[0][0] ) , x , k[2][0] , k[2][1] , t_dw , write_fit );

  cout << endl << " Saving \"sim.txt\" file for simulation curves. " << endl << endl ;

  return 0;
}



