// Simulating:  
// Neds reaction 4-13
//
#include <boost/array.hpp>
#include <boost/numeric/odeint.hpp>
#include <vector>
#include <iomanip>
#include <fstream>
#include <vector>
#include <cmath>
#include <iostream>
#include <sstream>
#include "Read.h"
#include <dlib/optimization.h>

using namespace std;
using namespace boost::numeric::odeint;

typedef boost::array< double , 11 > state_type;		// number of states / peaks 

  ofstream file3("sim.txt");                            // output file "sim.txt" for simulated fits

// passing params k1,k2 here and then calling odeint 
struct ode4
{
	double k1 ; double k1r ;
	double k2 ; double k3  ;
	double k4 ; double k5  ;
	double k6 ; double k7  ;

    ode4( 
	double m_k1 = 1.0  , double m_k1r = 0.05 , 
	double m_k2 = 0.05 , double m_k3  = 0.05 ,
	double m_k4 = 0.05 , double m_k5  = 0.05 ,
	double m_k6 = 0.05 , double m_k7  = 0.05 
	)
    	: k1( m_k1 ) , k1r( m_k1r ) , 
	  k2( m_k2 ) , k3 ( m_k3  ) ,   
	  k4( m_k4 ) , k5 ( m_k5  ) ,   
	  k6( m_k6 ) , k7 ( m_k7  )    
	{ }

	void operator()( const state_type &x , state_type &dxdt , double t ) 
	{
        // here are the diffeqs from the rxn mechanism:

    	dxdt[0] = -k1 * x[0] * x[1] + k1r * x[8] * x[2] ;
    	dxdt[1] = -k1 * x[0] * x[1] + k1r * x[8] * x[2] ;
    	dxdt[2] =  k1 * x[0] * x[1] - k1r * x[8] * x[2] ;

    	dxdt[3] =  k2 * x[8] - k7 * x[3] ;
    	dxdt[4] =  k2 * x[8] ;
    	dxdt[5] =  k4 * x[10] ;

    	dxdt[6] =  k5 * x[9] ;
    	dxdt[7] =  k6 * x[10] ;

    	dxdt[8]  =  -k2 * x[8] + k1 * x[0] * x[1] - k1r * x[8] * x[2] ;
    	dxdt[9]  =   k7 * x[3] - k3 * x[9] - k5 * x[9] ;
    	dxdt[10] =   k3 * x[9] - k4 * x[10] - k6 * x[10] ;

	}
} ;

//print statement to file "sim.txt"
void write_fit( const state_type &x , const double t )
{
  
  //double a = t * 10 ;
  //double b = floor(t * 10) ;		// floor statement only to print 0.1 units of time increments
  double a = t * 1 ;
  double b = floor(t * 1) ;		// floor statement only to print 1.0 units of time increments
  if (abs(b-a) < 0.001 ){          	// statement to correct for rounding errors b4 proceeding 
  file3 << fixed << setw(8) << setprecision(5) <<  t << ' ' << x[0] << ' ' << x[1] 
					             << ' ' << x[2] << ' ' << x[3]       
					             << ' ' << x[4] << ' ' << x[5]       
					             << ' ' << x[6] << ' ' << x[7]       
					             << ' ' << x[8] << ' ' << x[9]       
					             << ' ' << x[10]       
					             << endl;
  }
}

int main(int argc, char **argv)
{
  double t_dw = 0.01 		;					// time step used in runge-kutta stepping

  const char* FILE_NAMEB = "guess.txt";                                 // define file to data file to read-in
  std::vector< std::vector<double> > k = fileIn( FILE_NAMEB );          // push data file in matrix A / see Read.h 
  cout << endl << " Reading \"guess.txt\" file for: " << endl ;
  cout << " initial rate constants " << endl ;
  cout << " initial states " << endl ;
  cout << " begin_time end_time " << endl << endl ;
  print_dat (k) ;							// print out guess.txt contents

  state_type x = {	k[1][0], k[1][1], k[1][2] , k[1][3], 
	       		k[1][4], k[1][5], k[1][6] , k[1][7],	
	       		k[1][8], k[1][9], k[1][10] 	
		 } ; 							// initial conditions/state (peak heights at time = 0 )

  typedef runge_kutta4< state_type > stepper_type;
  integrate_const( stepper_type() , 
	ode4( k[0][0] , k[0][1], k[0][2] , k[0][3] , k[0][4], k[0][5] , k[0][6] , k[0][7] ) , 
	      x , k[2][0] , k[2][1] , t_dw , write_fit ) ;

  cout << endl << " Saving \"sim.txt\" file for simulation curves. " << endl << endl ;

  return 0;
}



